

Imports System.ComponentModel
Imports System.Diagnostics
Imports System.Runtime.InteropServices


' ------------------------------------------------------------- constants for the multimedia Timer's event types
Public Enum TimerMode
    OneShot
    Periodic
End Enum

' ------------------------------------------------------------- information about the multimedia Timer's capabilities
<StructLayout(LayoutKind.Sequential)> _
Public Structure TimerCaps
    Public periodMin As Integer
    Public periodMax As Integer
End Structure


' -------------------------------------------------------------- the Windows multimedia timer
Public NotInheritable Class Timer
    Implements IComponent

    ' ---------------------------------------------------------- the method called by Windows when a timer event occurs.
    Private Delegate Sub TimeProc(ByVal id As Integer, _
                                  ByVal msg As Integer, _
                                  ByVal user As Integer, _
                                  ByVal param1 As Integer, _
                                  ByVal param2 As Integer)

    ' ---------------------------------------------------------- methods that raise events.
    Private Delegate Sub EventRaiser(ByVal e As EventArgs)


    <DllImport("winmm.dll")> _
    Private Shared Function timeGetDevCaps(ByRef caps As TimerCaps, ByVal sizeOfTimerCaps As Integer) As Integer
    End Function

    <DllImport("winmm.dll")> _
    Private Shared Function timeSetEvent(ByVal delay As Integer, ByVal resolution As Integer, ByVal proc As TimeProc, ByVal user As Integer, ByVal mode As Integer) As Integer
    End Function

    <DllImport("winmm.dll")> _
    Private Shared Function timeKillEvent(ByVal id As Integer) As Integer
    End Function

    Private Const TIMERR_NOERROR As Integer = 0

    Private timerID As Integer
    Private m_mode As TimerMode
    Private m_period As Integer
    Private m_resolution As Integer
    Private timeProcPeriodic As TimeProc
    Private timeProcOneShot As TimeProc
    Private tickRaiser As EventRaiser
    Private running As Boolean = False
    Private m_disposed As Boolean = False
    Private Shared caps As TimerCaps

    ' ---------------------------------------------------------------- For implementing IComponent.
    Private m_site As ISite = Nothing
    ' ---------------------------------------------------------------- ISynchronizeInvoke object for marshaling events
    Private m_synchronizingObject As ISynchronizeInvoke = Nothing
 

    Public Event Started As EventHandler
    Public Event Stopped As EventHandler
    Public Event Tick As EventHandler


    Shared Sub New()
        ' ------------------------------------------------------------ get multimedia timer capabilities
        timeGetDevCaps(caps, Marshal.SizeOf(caps))
    End Sub

    ' ---------------------------------------------------------------- Initializes Timer with specified IContainer
    Public Sub New(ByVal container As IContainer)
        '''
        ''' Required for Windows.Forms Class Composition Designer support
        '''
        container.Add(Me)
        Initialize()
    End Sub

    Public Sub New()
        Initialize()
    End Sub

    Protected Overrides Sub Finalize()
        Try
            If IsRunning Then
                timeKillEvent(timerID)
            End If
        Finally
            MyBase.Finalize()
        End Try
    End Sub

    ' -------------------------------------------------------------------- initialize timer with default values
    Private Sub Initialize()
        Me.m_mode = TimerMode.Periodic
        Me.m_period = Capabilities.periodMin
        Me.m_resolution = 1
        running = False
        timeProcPeriodic = New TimeProc(AddressOf TimerPeriodicEventCallback)
        timeProcOneShot = New TimeProc(AddressOf TimerOneShotEventCallback)
        tickRaiser = New EventRaiser(AddressOf OnTick)
    End Sub



    Public Sub TimerStart()
        If m_disposed Then Throw New ObjectDisposedException("Timer")
        If IsRunning Then Return

        ' If the periodic event callback should be used.
        If Mode = TimerMode.Periodic Then
            ' Create and start timer.
            timerID = timeSetEvent(Period, Resolution, timeProcPeriodic, 0, Mode)
        Else
            ' Else the one shot event callback should be used.
            ' Create and start timer.
            timerID = timeSetEvent(Period, Resolution, timeProcOneShot, 0, Mode)
        End If

        ' If the timer was created successfully.
        If timerID <> 0 Then
            running = True

            If SynchronizingObject IsNot Nothing AndAlso SynchronizingObject.InvokeRequired Then
                SynchronizingObject.BeginInvoke(New EventRaiser(AddressOf OnStarted), New Object() {EventArgs.Empty})
            Else
                OnStarted(EventArgs.Empty)
            End If
        Else
            Throw New TimerStartException("Unable to start multimedia Timer.")
        End If
    End Sub


    Public Sub TimerStop()
        If m_disposed Then Throw New ObjectDisposedException("Timer")
        If Not running Then Return

        Dim result As Integer = timeKillEvent(timerID)
        Debug.Assert(result = TIMERR_NOERROR)
        running = False

        If SynchronizingObject IsNot Nothing AndAlso SynchronizingObject.InvokeRequired Then
            SynchronizingObject.BeginInvoke(New EventRaiser(AddressOf OnStopped), New Object() {EventArgs.Empty})
        Else
            OnStopped(EventArgs.Empty)
        End If
    End Sub

#Region "Callbacks"

    ' Callback method called by the Win32 multimedia timer when a timer
    ' periodic event occurs.
    Private Sub TimerPeriodicEventCallback(ByVal id As Integer, ByVal msg As Integer, ByVal user As Integer, ByVal param1 As Integer, ByVal param2 As Integer)
        If m_disposed Then Return
        '
        If m_synchronizingObject IsNot Nothing Then
            m_synchronizingObject.BeginInvoke(tickRaiser, New Object() {EventArgs.Empty})
        Else
            OnTick(EventArgs.Empty)
        End If
    End Sub

    ' Callback method called by the Win32 multimedia timer when a timer
    ' one shot event occurs.
    Private Sub TimerOneShotEventCallback(ByVal id As Integer, ByVal msg As Integer, ByVal user As Integer, ByVal param1 As Integer, ByVal param2 As Integer)
        If m_disposed Then Return
        '
        If m_synchronizingObject IsNot Nothing Then
            m_synchronizingObject.BeginInvoke(tickRaiser, New Object() {EventArgs.Empty})
            TimerStop()
        Else
            OnTick(EventArgs.Empty)
            TimerStop()
        End If
    End Sub

#End Region

#Region "Event Raiser Methods"

    Private Sub OnDisposed(ByVal e As EventArgs)
        RaiseEvent Disposed(Me, e)
    End Sub

    Private Sub OnStarted(ByVal e As EventArgs)
        RaiseEvent Started(Me, e)
    End Sub

    Private Sub OnStopped(ByVal e As EventArgs)
        RaiseEvent Stopped(Me, e)
    End Sub

    Private Sub OnTick(ByVal e As EventArgs)
        RaiseEvent Tick(Me, e)
    End Sub

#End Region


#Region "Properties"

    ' -------------------------------------------- Gets or sets the object used to marshal event-handler calls.
    Public Property SynchronizingObject() As ISynchronizeInvoke
        Get
            If m_disposed Then Throw New ObjectDisposedException("Timer")
            Return m_synchronizingObject
        End Get
        Set(ByVal value As ISynchronizeInvoke)
            If m_disposed Then Throw New ObjectDisposedException("Timer")
            m_synchronizingObject = value
        End Set
    End Property


    ' --------------------------------------------- Gets or sets the time between Tick events.
    Public Property Period() As Integer
        Get
            If m_disposed Then Throw New ObjectDisposedException("Timer")
            Return m_period
        End Get
        Set(ByVal value As Integer)
            If m_disposed Then
                Throw New ObjectDisposedException("Timer")
            ElseIf value < Capabilities.periodMin OrElse value > Capabilities.periodMax Then
                Throw New ArgumentOutOfRangeException("Period", value, "Multimedia Timer period out of range.")
            End If
            m_period = value
            If IsRunning Then
                TimerStop()
                TimerStart()
            End If
        End Set
    End Property


    ' ------------------------------------------------------------------ Gets or sets the timer resolution.
    Public Property Resolution() As Integer
        Get
            If m_disposed Then Throw New ObjectDisposedException("Timer")
            Return m_resolution
        End Get
        Set(ByVal value As Integer)
            If m_disposed Then
                Throw New ObjectDisposedException("Timer")
            ElseIf value < 0 Then
                Throw New ArgumentOutOfRangeException("Resolution", value, "Multimedia timer resolution out of range.")
            End If
            m_resolution = value
            If IsRunning Then
                TimerStop()
                TimerStart()
            End If
        End Set
    End Property

    ' ------------------------------------------------------------------ Gets the timer mode.
    Public Property Mode() As TimerMode
        Get
            If m_disposed Then Throw New ObjectDisposedException("Timer")
            Return m_mode
        End Get
        Set(ByVal value As TimerMode)
            If m_disposed Then Throw New ObjectDisposedException("Timer")
            m_mode = value
            If IsRunning Then
                TimerStop()
                TimerStart()
            End If
        End Set
    End Property

    ' ---------------------------------------------------------- Gets a value indicating whether the Timer is running.
    Public ReadOnly Property IsRunning() As Boolean
        Get
            Return running
        End Get
    End Property

    ' ---------------------------------------------------------- Gets the timer capabilities.
    Public Shared ReadOnly Property Capabilities() As TimerCaps
        Get
            Return caps
        End Get
    End Property

#End Region


#Region "IComponent Members"

    Public Event Disposed As System.EventHandler

    Public Property Site() As ISite Implements IComponent.Site
        Get
            Return m_site
        End Get
        Set(ByVal value As ISite)
            m_site = value
        End Set
    End Property

#End Region

#Region "IDisposable Members"

    ' --------------------------------------------------------------- Frees timer resources.
    Public Sub Dispose() Implements IDisposable.Dispose
        If m_disposed Then Return
        m_disposed = True
        If running Then
            timeKillEvent(timerID)
        End If
        OnDisposed(EventArgs.Empty)
    End Sub

#End Region

    Public Event Disposed1(ByVal sender As Object, ByVal e As System.EventArgs) Implements System.ComponentModel.IComponent.Disposed

End Class


' ------------------------------------------------- The exception that is thrown when a timer fails to start.
Public Class TimerStartException
    Inherits ApplicationException
    '
	Public Sub New(message As String)
		MyBase.New(message)
	End Sub
End Class
